package org.openxma.dsl.generator.component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Set;

import org.eclipse.xpand2.output.FileHandle;
import org.eclipse.xpand2.output.PostProcessor;

import com.google.common.collect.Sets;

public class JavaImportPostProcessor implements PostProcessor {
	public final static String VAR_NAME = "__javaImportTool";
	private Set<String> imports = Sets.newTreeSet();
	private String lastInsertMarker = null;

	public void beforeWriteAndClose(FileHandle impl) {
		if (lastInsertMarker != null) {
			String string = impl.getBuffer().toString();
			impl.setBuffer(new StringBuilder(string.replace(lastInsertMarker,
					getImportSection())));
		}
	}

	protected CharSequence getImportSection() {
		StringBuilder sb = new StringBuilder();
		ArrayList<String> list = new ArrayList<String>(imports);
		Collections.sort(list);
		for (String name : list) {
			if (name.indexOf('.') > -1) {
				sb.append("import ").append(name).append(";\n");
			}
		}
		return sb;
	}

	public void afterClose(FileHandle impl) {
		lastInsertMarker = null;
	}

	public String addImportsHere() {
		if (lastInsertMarker != null)
			throw new IllegalStateException(
					"Multiple import sections? Did you forget to register the JavaImportsTool as postprocessor?");
		imports.clear();
		lastInsertMarker = "_Placeholder_for_a_lazy_import_section_"
				+ System.currentTimeMillis() + "___";
		return lastInsertMarker;
	}

	public String registerImport(String qualifiedName) {
		imports.add(qualifiedName);
		String[] strings = qualifiedName.split("\\.");
		return lastInsertMarker !=null && strings.length > 1 ? strings[strings.length - 1] : qualifiedName;
	}

}
