package org.openxma.dsl.generator.impl;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConfiguration;

public abstract class AbstractGenerator implements Generator {
    protected final Log logger = LogFactory.getLog(getClass());
    private GeneratorConfiguration configuration;

    public void init(GeneratorConfiguration generatorConfiguration) {
        this.configuration = generatorConfiguration;
    }

    public GeneratorConfiguration getConfiguration() {
        return configuration;
    }

    protected String toRelativePath(URI uri) {
        String relativePath = null;
        if (uri.isPlatformResource()) {
            IFolder folder = EcorePlugin.getWorkspaceRoot().getFolder((IPath) toPath(uri));
            relativePath = folder.getProjectRelativePath().toString() + File.separator;
        } else {
            relativePath = new File(getConfiguration().getProjectFolder().toFileString()).toURI().relativize(
                    new File(uri.toFileString()).toURI()).getPath();
            relativePath = relativePath + File.separator;
        }
        return relativePath;

    }

    protected Object toPath(URI uri) {
        return new Path(uri.toPlatformString(true));
    }

}
