package org.openxma.dsl.generator.impl.substrategy;

import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.common.Strings;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.ValueObject;
import org.openxma.dsl.generator.helper.BasicType;
import org.openxma.dsl.generator.helper.BasicTypeHelper;
import org.openxma.dsl.generator.helper.EventExtension;
import org.openxma.dsl.generator.helper.WidgetExtension;
import org.openxma.dsl.generator.impl.DomPageLogicAttachmentStrategy;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.StylePropertyTableCustomizer;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.scoping.PomElementCollector;
import org.openxma.dsl.pom.util.StyleAccess;

import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAPagingControl;
import at.spardat.xma.guidesign.XMATable;
import at.spardat.xma.guidesign.XMATableColumn;

public class TableCustomizerStrategy {

    public List<XMATable> getXMATables(XMAPage page) {
        return PomElementCollector.getElementsOfXMAPage(page,XMATable.class);
    }

    public Set<XMATable> getTablesWithCustomizer(XMAPage page) {
        Set<XMATable> pageableTables = getPageableTables(page);
        Set<XMATable> resultSet = new LinkedHashSet<XMATable>();
        for (XMATable xmaTable : pageableTables) {
            Table table = getTableByXMATable(xmaTable);
            String tableCustomizerInterface = null;
            tableCustomizerInterface = getTableCustomizerInterface(table);
            if (tableCustomizerInterface != null) {
                resultSet.add(xmaTable);
            }
        }
        return resultSet;
    }

    public Set<XMATable> getPageableTables(XMAPage page) {
        List<XMATable> tablesOfPage = getXMATables(page);
        Set<XMATable> resultSet = new LinkedHashSet<XMATable>();
        for (XMATable xmaTable : tablesOfPage) {
            Table table = getTableByXMATable(xmaTable);
            if (table != null) {
                if (table.getPageable() instanceof TrueLiteral) {
                    resultSet.add(xmaTable);
                }
            }
        }
        return resultSet;

    }


    public void genCreateWidgetsCodeClient(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, XMAPage page, StringBuilder buf) {
        Set<XMATable> tablesWithCustomizer = getTablesWithCustomizer(page);
        for (XMATable xmaTable : tablesWithCustomizer) {

            String tableCustomizerName = xmaTable.getNamInstance() + "_customizer";
            XMAPagingControl pc = getPagingControl(xmaTable);
            String pcNamInstance = pc.getNamInstance();
            String pcNamInstanceW = pc.getNamInstance() + "W";
            Table table = getTableByXMATable(xmaTable);
            String tableCustomizerInterface = getTableCustomizerInterface(table);

            buf.append("\n        /* setting icon for tableCustomizer on control " + pcNamInstanceW + " */");
            buf.append("\n        " + pcNamInstanceW + ".setCusomizeIcon(org.openxma.addons.ui.table.customizer.common.TableCustomizerUISettingsFactory.createUISettings(this,getComposite()).getCustomizerImage());");

            buf.append("\n        /* setting default icons for pagingcontrol */");
            buf.append("\n        " + pcNamInstanceW + ".setBackIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/backward.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setFastBackIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/fastbackward.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setFastNextIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/fastforward.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setNextIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/forward.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setReloadIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/reload.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setStartIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/start.png\"));");
            buf.append("\n        " + pcNamInstanceW + ".setEndIcon(getComponent().getImage(\"at/spardat/xma/mdl/paging/icons/end.png\"));");

            String xmaTableName = xmaTable.getNamInstance();
            buf.append("\n        /* Table customizer for " + xmaTableName + " */");
            buf.append("\n        " + tableCustomizerInterface + " " + tableCustomizerName + " = (" + tableCustomizerInterface + ") getTypedComponent().getSession().getComponent(" + tableCustomizerInterface + ".COMPONENT_NAME);");
            buf.append("\n        " + tableCustomizerName + ".setTableLayoutManager(" + xmaTable.getNamInstance() + "WLM);");
            buf.append("\n        " + tableCustomizerName + ".setTableWMClient(" + xmaTable.getNamInstance() + ");");
            buf.append("\n        " + tableCustomizerName + ".setFQTableName(\"" + page.getComponent().getNamPackage() + "." + page.getNamClass() + "." + xmaTableName + "\");");
            buf.append("\n        Map<String, ColumnMetaData> " + xmaTableName + "_columnMap = new HashMap<String, ColumnMetaData>();");
            for (Object obj : xmaTable.getColumn()) {
                XMATableColumn xmaColumn = (XMATableColumn) obj;
                TableColumn xmadslColumn = (TableColumn)WidgetExtension.getSourceModelElementProperty(xmaColumn);                
                if (xmadslColumn == null || xmadslColumn.getFeature() == null || xmaColumn.getBdValidator()==null) {
                    continue; // Do not add pure UI columns (without linked
                              // attributes)
                }
                EObject attributeHolder = xmadslColumn.getFeature().getAttributeHolder();
                Attribute attribute = null;
                boolean isValueObject = false;
                if (attributeHolder instanceof FeatureReference) {
                    FeatureReference featureReference = (FeatureReference) attributeHolder;
                    if (featureReference.getSource() instanceof Entity) {
                        attribute = featureReference.getAttribute();
                    }
                } else if (attributeHolder instanceof Attribute) {
                    if (attributeHolder.eContainer() instanceof Entity) {
                        attribute = (Attribute) attributeHolder;
                    } else if (attributeHolder.eContainer() instanceof ValueObject) {
                        attribute = (Attribute) attributeHolder;
                        isValueObject = true;
                    } else if (attributeHolder.eContainer() instanceof DataView) {
                        attribute = (Attribute) attributeHolder;                        
                    }
                    
                }
                List<String> numberTypes = Arrays.asList(new String[]{"Long","Integer","BigDecimal", "Double", "Float"});

                if (attribute != null) {
                    String typeName = null;
                    BasicType basicType = BasicTypeHelper.determineBasicType(attribute);
                    switch (basicType) {
                    case STRING:
                        typeName = "String";
                        break;
                    case I18N_ENUMERATION:
                        typeName= "Enum";
                        break;
                    case BOOLEAN:
                        typeName="Boolean";
                        break;
                    case DATE:
                        typeName="Date";
                        break;
                    case NUMBER:
                        if (numberTypes.contains(attribute.getDataTypeName())) {
                            typeName = attribute.getDataTypeName();
                        } else {
                            ValidatorReference rv = BasicTypeHelper.determineRootValidator(attribute);
                            if (rv.eContainer() instanceof SimpleType) {
                                typeName = ((org.openxma.dsl.core.model.impl.SimpleTypeImpl) rv.eContainer()).getTypeDefinition().getInstanceType();
                                if (typeName.indexOf(".")>0){
                                    typeName = typeName.substring(typeName.lastIndexOf(".")+1);
                                }
                            } else if (attribute.getType().getDataType() instanceof SimpleType){
                                DataTypeAndTypeParameter definition = ((org.openxma.dsl.core.model.impl.SimpleTypeImpl) attribute.getType().getDataType()).getTypeDefinition().getDefinition();
                                if (definition != null) {
                                    typeName = definition.getDataType().getName();                                    
                                } else {
                                    throw new IllegalArgumentException("type not supported: " + attribute.getDataTypeName());                                    
                                }
                            } else {
                                throw new IllegalArgumentException("type not supported: " + attribute.getDataTypeName());
                            }
                        }                        
                        break;
                    case TIME_STAMP:
                        typeName = "Timestamp";
                        break;
                    case CUSTOM:
                        throw new IllegalArgumentException("type not supported: " + attribute.getDataTypeName());                        
                    default:       
                        typeName = attribute.getDataTypeName();
                        break;
                    }
                    if (isValueObject) {
                        String valuePrefix = ((at.spardat.xma.guidesign.impl.BusinessDataImpl) xmaColumn.getDataAttribute().eContainer()).getNamInstance();
                        buf.append("\n        " + xmaTableName + "_columnMap.put(\"" + valuePrefix + "." + attribute.getName() + "\",new ColumnMetaData(\"" + (valuePrefix + "." + attribute.getName()).toUpperCase()
                                + "\", getGenPageMessages().getString(\"" + xmaColumn.getNamRscKeyLabel() + "\"), " + xmaColumn.getNamInstance().toUpperCase() + ",\"" + typeName + "\"));");
                    } else {
                        buf.append("\n        " + xmaTableName + "_columnMap.put(\"" + attribute.getName() + "\",new ColumnMetaData(\"" + attribute.getName().toUpperCase() + "\", getGenPageMessages().getString(\"" + xmaColumn.getNamRscKeyLabel()
                                + "\"), " + xmaColumn.getNamInstance().toUpperCase() + ",\"" + typeName+ "\"));");
                    }
                }
            }
            buf.append("\n        " + tableCustomizerName + ".setColumnMap(" + xmaTableName + "_columnMap);");
            buf.append("\n        " + tableCustomizerName + ".setAppCode(getAppCode());");
            buf.append("\n        " + tableCustomizerName + ".setPagingWMClient(" + xmaTableName + "_pagingControl);");
            buf.append("\n        " + tableCustomizerName + ".init();");
            buf.append("\n        " + xmaTable.getNamInstance() + "_customizerComponentId.set((short)" + tableCustomizerName + ".getId());");
            buf.append("\n        " + pcNamInstance + ".setCustomizer(" + tableCustomizerName + ");");
            buf.append("\n");
        }

    }

    public XMAPagingControl getPagingControl(XMATable xmaTable) {
        return EventExtension.getPagingControlForXMATable(xmaTable);
    }

    public void genAdditionalImportsClient(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, XMAPage xmaPage, LinkedHashSet<String> imports) {
        List<XMATable> tablesOfPage = getXMATables(xmaPage);
        for (XMATable xmaTable : tablesOfPage) {
            GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaTable);
            if (!(guiElement instanceof Table)) continue;
            Table table = (Table)guiElement;
            if (table.getPageable() instanceof TrueLiteral) {
                Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(table);
                if (styleSpecificationProperties != null) {
                    StylePropertyTableCustomizer customizerProp = getStylePropertyTableCustomizer(styleSpecificationProperties);
                    if (customizerProp != null) {
                        String customizerType = customizerProp.getTableCustomizer().getInstanceType();
                        imports.add(customizerType);
                        imports.add("java.util.Map");
                        imports.add("java.util.HashMap");
                        imports.add("org.openxma.addons.ui.table.customizer.common.ColumnMetaData");
                    }
                }
            }
        }
    }

    public void genAdditionalImportsServer(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, XMAPage xmaPage, LinkedHashSet<String> imports) {
        List<XMATable> tablesOfPage = getXMATables(xmaPage);
        for (XMATable xmaTable : tablesOfPage) {
            GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaTable);
            if (!(guiElement instanceof Table)) continue;
            Table table = (Table)guiElement;
            if (table.getPageable() instanceof TrueLiteral) {
                Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(table);
                if (styleSpecificationProperties != null) {
                    StylePropertyTableCustomizer customizerProp = getStylePropertyTableCustomizer(styleSpecificationProperties);
                    if (customizerProp != null) {
                        imports.add("org.openxma.dsl.platform.query.QueryObject");
                        imports.add("at.spardat.xma.session.XMASessionServer");
                        imports.add(getTableCustomizerCompName(customizerProp.getTableCustomizer().getInstanceType(),true));
                    }
                }
            }
        }
    }

    public void genAdditionalMemberVariablesClient(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, XMAPage xmaPage, StringBuilder stringBuilder) {
    }

    public void genAdditionalMemberVariablesServer(
            DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy,
            XMAPage xmaPage, StringBuilder stringBuilder) {

    }

    protected String getTableCustomizerCompName(String tableCustomizerInterfaceName, boolean fullQualified) {
        if (tableCustomizerInterfaceName == null) {
            throw new IllegalArgumentException("The value of tableCustomizerInterfaceName must not be null");
        }
        String tableCustomizername;
        int lastDotIndex = tableCustomizerInterfaceName.lastIndexOf('.');
        if (lastDotIndex >= 0) {
            tableCustomizername = tableCustomizerInterfaceName.substring(lastDotIndex+1);
        } else {
            tableCustomizername = tableCustomizerInterfaceName;
        }
        tableCustomizername+="Server";
        if (fullQualified && lastDotIndex >= 0) {
            String clientPackage = tableCustomizerInterfaceName.substring(0,lastDotIndex);
            if (clientPackage.endsWith(".client")) {
                int clientDotIndex = clientPackage.lastIndexOf(".client");
                String serverPackage = clientPackage.substring(0,clientDotIndex)+".server.";
                tableCustomizername = serverPackage+tableCustomizername;
            } else {
                tableCustomizername = clientPackage+"."+tableCustomizername;
            }
        }
        return tableCustomizername;
    }

    public void genGetQueryObject(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, XMAPage page, StringBuilder buffer) {

         List<XMATable> tablesOfPage = getXMATables(page);
            for (XMATable xmaTable : tablesOfPage) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaTable);
                if (!(guiElement instanceof Table)) continue;
                Table table = (Table)guiElement;
                String tableCustomizername = null;
                if (table.getPageable() instanceof TrueLiteral) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(table);
                    if (styleSpecificationProperties != null) {
                        StylePropertyTableCustomizer customizerProp = getStylePropertyTableCustomizer(styleSpecificationProperties);
                        if (customizerProp != null) {
                            tableCustomizername = getTableCustomizerCompName(customizerProp.getTableCustomizer().getInstanceType(),false);
                        }
                    }
                    if (tableCustomizername == null) {
                        continue;
                    }
                    XMAPagingControl pc = getPagingControl(xmaTable);
                    buffer.append(
                        "\n    /**\n"+
                        "     * Gets the QueryObject for the table <tt>"+xmaTable.getNamInstance()+"</tt>\n"+
                        "     */\n"+
                        "     public QueryObject get"+Strings.capitalize(xmaTable.getNamInstance())+"QueryObject() {\n"+
                        "         int offset = "+pc.getNamInstance()+".getOffset();\n"+
                        "         short pageSize = "+pc.getNamInstance()+".getPageSize();\n"+

                        "         short sortingColumn = "+pc.getNamInstance()+".getSortingColumn();\n"+
                        "         boolean isAscending = "+pc.getNamInstance()+".getAscending();\n"+
                        "         XMASessionServer xmaSessionServer = (XMASessionServer)XMASessionServer.getXMASession();\n"+
                        "         "+tableCustomizername+" tableCustomizer = ("+tableCustomizername+")xmaSessionServer.getComponent(Short.valueOf("+xmaTable.getNamInstance()+"_customizerComponentId.getEncodedValue()));\n"+
                        "         QueryObject filterQueryForTable = null;\n"+
                        "         if (sortingColumn >-1) {\n"+
                        "             filterQueryForTable = tableCustomizer.getQueryObject(sortingColumn,isAscending);\n"+
                        "         } else {\n"+
                        "             filterQueryForTable = tableCustomizer.getQueryObject();\n"+
                        "         }\n"+
                        "         filterQueryForTable.setFirstResult(offset);\n"+
                        "         filterQueryForTable.setMaxResults(pageSize);\n"+
                        "         return filterQueryForTable;\n"+
                        "     }\n");
                }
            }

    }

    public Table getTableByXMATable(XMATable xmaTable) {
        Table retTable = null;
        GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaTable);
        if (guiElement instanceof Table){
            retTable = (Table)guiElement;
        }
        return retTable;
    }

    public String getTableCustomizerInterface(Table table) {
        String tableCustomizerInterface = null;
        Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(table);
        if (styleSpecificationProperties != null) {
            StylePropertyTableCustomizer customizerProp = getStylePropertyTableCustomizer(styleSpecificationProperties);
            if (customizerProp != null) {
                tableCustomizerInterface = customizerProp.getTableCustomizer().getInstanceType();
                int lastDotIndex = tableCustomizerInterface.lastIndexOf('.');
                if (lastDotIndex >= 0) {
                    tableCustomizerInterface = tableCustomizerInterface.substring(lastDotIndex+1);
                }
            }
        }
        return tableCustomizerInterface;
    }

    private StylePropertyTableCustomizer getStylePropertyTableCustomizer(
            Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyTableCustomizer)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_TABLE_CUSTOMIZER);
    }
}
